from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
import smtplib

class Email:
    """
        This class represents an email generated by the Emailer library.
    """

    def __init__(self, host: str, port: int, sender) -> None:
        """
        Initialize an Email object.

        Args:
            host (str): The host of the email server.
            port (int): The port number of the email server.
            sender (str): The email address of the sender.
        """
        self.host = host
        self.port = port
        self.sender = sender

    def add_html_body_from_file(self, file_path: str) -> None:
        """
        Add an HTML body to the email from a file.

        Args:
            file_path (str): The path to the file containing the HTML body of the email.
        """
        with open(file_path, "r") as file:
            self.html_body = file.read()

    def add_format_variables(self, format_variables: dict) -> None:
        """
        Add variables to be used for formatting the email body.

        Args:
            format_variables (dict): A dictionary of variables to be used for formatting the email body.
        """
        self.format_variables = format_variables

    def add_html_body(self, html_body: str) -> None:
        """
        Add an HTML body to the email.

        Args:
            html_body (str): The HTML body of the email.
        """
        self.html_body = html_body

    def add_plain_body(self, plain_body: str) -> None:
        """
        Add a plain text body to the email.

        Args:
            plain_body (str): The plain text body of the email.
        """
        self.plain_body = plain_body

    def __repr__(self):
        return f"Email(host={self.host}, port={self.port})"

    def __decide_body__(self):
        """
        Decide the type of body to use based on the available attributes.

        Returns:
            str: The type of body to use. Possible values are "both", "plain", or "html".

        Raises:
            ValueError: If no body is specified.
        """
        if hasattr(self, "plain_body") and hasattr(self, "html_body"):
            return "both"
        if hasattr(self, "plain_body"):
            return "plain"
        if hasattr(self, "html_body"):
            return "html"
        raise ValueError("No body specified.")

    def __format_body__(self, body) -> str:
        """
        Formats the body of the email by replacing variables with their corresponding values.

        Args:
            body (str): The body of the email.

        Returns:
            str: The formatted body of the email.
        """
        if self.format_variables is not None:
            return body.format(**self.format_variables)
        else:
            return body

    def send(self, recipient: str, subject: str) -> None:
        """
        Sends an email with the specified sender, recipient, and subject.

        Args:
            recipient (str): The email address of the recipient.
            subject (str): The subject of the email.

        Returns:
            None
        """

        if not hasattr(self, "html_body") and not hasattr(self, "plain_body"):
            raise ValueError("No body specified.")

        body_type = self.__decide_body__()

        if body_type == "both":
            plain_body = self.__format_body__(self.plain_body)
            html_body = self.__format_body__(self.html_body)
            msg = MIMEMultipart("alternative")
            msg.attach(MIMEText(plain_body, "plain"))
            msg.attach(MIMEText(html_body, "html"))
        else:
            body = self.__format_body__(getattr(self, body_type + "_body"))
            msg = MIMEText(body, body_type)

        msg["Subject"] = subject
        msg["From"] = self.sender
        msg["To"] = recipient

        s = smtplib.SMTP(self.host, self.port)
        s.sendmail(self.sender, recipient, msg.as_string())
        s.quit()

